/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2011
 *    Denis Klimentjew <klimentjew@informatik.uni-hamburg.de>
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef CAMCUBE_H_
#define CAMCUBE_H_

//
#include <stdio.h>
#include <stdlib.h>

#include "../include/pmd_camcube/pmdsdk2.h"

#include <ros/ros.h>

#define SOURCE_PLUGIN "/opt/ros/fuerte/stacks/pmd_camcube/plugins/camcube3"
#define SOURCE_PARAM ""
#define PROC_PLUGIN "/opt/ros/fuerte/stacks/pmd_camcube/plugins/camcubeproc"
#define PROC_PARAM ""

using namespace std;

// ROS messages
#include <sensor_msgs/PointCloud.h>
#include <sensor_msgs/PointCloud2.h>
#include <sensor_msgs/Image.h>

#include <ros/ros.h>
#include <ros/package.h>
#include <image_transport/image_transport.h>
#include <cv_bridge/CvBridge.h>

namespace pmd_camcube
{
  class CamcubeDriver
  {
    public:
      /** \brief Constructor */
      CamcubeDriver (ros::NodeHandle comm_nh, ros::NodeHandle param_nh);
      virtual ~CamcubeDriver ();

      void processGrayscaleAndDepth (PMDHandle hnd);
 
      void showSerialNumber (PMDHandle hnd);
      
      void publishImg (PMDHandle hnd, IplImage *img);

      void publish ();


    private:
      
      /** \brief ROS publishers. */
      image_transport::Publisher image_pub_;
      ros::Publisher pub_depth_points_, pub_depth_points2_;

      /** \brief Camera parameters. */
      int width_;
      int height_;

      /** \brief Image data. */
      sensor_msgs::CvBridge bridge_;
      /** \brief PointCloud data. */
      sensor_msgs::PointCloud cloud_;
      /** \brief PointCloud2 data. */
      sensor_msgs::PointCloud2 cloud2_;
      /** \brief Camera info data. */


    public:
      PMDHandle hnd; //connection handle
      int res;
      char err[128];
      PMDDataDescription dd;
      IplImage *img;
  };

}

#endif //CAMCUBE_H_
