/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2011
 *    Denis Klimentjew <klimentjew@informatik.uni-hamburg.de>
 *   
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "pmd_camcube/camcube.h"
#include "camcube.cpp"
#include <signal.h>

PMDHandle hnd; //connection handle

void ctrlc(int s)
{
    cout << "ctrl-c pressed" << endl;
    pmdClose (hnd);
    ros::shutdown();
    signal(SIGINT, SIG_DFL);
}
      
int main (int argc, char **argv)
{
  // Init ROS
  ros::init (argc, argv, "camcube_node"); 

  ros::NodeHandle comm_nh ("camera"); // for topics, services
  ros::NodeHandle param_nh ("~");     // for parameters

  // Init CamCube
  int res;
  char err[128];
  PMDDataDescription dd;
  IplImage *img = cvCreateImage(cvSize(200,200), IPL_DEPTH_8U, 1);
  
  res = pmdOpen (&hnd, SOURCE_PLUGIN, SOURCE_PARAM, PROC_PLUGIN, PROC_PARAM); // connect to camera

  if (res != PMD_OK)
    {
      pmdGetLastError (0, err, 128);
      ROS_INFO("Could not connect: %s\n", err); 
      return 1;
    }


  res = pmdUpdate (hnd);

  if (res != PMD_OK)
    {
      pmdGetLastError (hnd, err, 128);
      ROS_WARN ("Could transfer data: %s\n", err);
      pmdClose (hnd);
      return 1;
    }


  res = pmdGetSourceDataDescription (hnd, &dd);
  
  if (res != PMD_OK)
    {
      pmdGetLastError (hnd, err, 128);
      ROS_WARN ("Could get data description: %s\n", err);
      pmdClose (hnd);
      return 1;
    }

  if (dd.subHeaderType != PMD_IMAGE_DATA)
    {
      ROS_WARN ("Source data is not an image!\n");
      pmdClose (hnd);
      return 1;
    }
    
 // Read the device_id parameter from the server
  int device_id;
  param_nh.param ("device_id", device_id, argc > 1 ? atoi (argv[1]) : 0);

  // Create the CamcubeDriver object
  pmd_camcube::CamcubeDriver pmd (comm_nh, param_nh);
  pmd.showSerialNumber (hnd);

  ros::Duration r (0.0001);
  
  while (ros::ok ())
  {
    signal(SIGINT, ctrlc);
    pmd.processGrayscaleAndDepth (hnd);
    pmd.publishImg (hnd, img);
    ros::spinOnce ();
    r.sleep ();
  }

  return (0);
}

